@@ -2,8 +2,8 @@
 package Memoize::Expire;
 # require 5.00556;
 use Carp;
-$DEBUG = 0;
-$VERSION = '1.00';
+use vars qw($DEBUG);
+$VERSION = '1.04';
 
 # This package will implement expiration by prepending a fixed-length header
 # to the font of the cached data.  The format of the header will be:
@@ -120,6 +120,10 @@ sub _get_header  {
 
 1;
 
+__END__
+
+=pod
+
 =head1 NAME 
 
 Memoize::Expire - Plug-in module for automatic expiration of memoized values
@@ -311,10 +315,10 @@ the underlying cache so that the user can specify that the cache is
 also persistent or that it has some other interesting semantics.  The
 example above demonstrates how to do this, as does C<Memoize::Expire>.
 
-Another sample module, C<Memoize::Saves>, is included with this
-package.  It implements a policy that allows you to specify that
-certain function values would always be looked up afresh.  See the
-documentation for details.
+Another sample module, L<Memoize::Saves>, is available in a separate
+distribution on CPAN.  It implements a policy that allows you to
+specify that certain function values would always be looked up afresh.
+See the documentation for details.
 
 =head1 ALTERNATIVES
 
@@ -344,12 +348,12 @@ with a lifetime of ten seconds, and you store it at 12:00:00.998 on a
 certain day.  Memoize will look at the clock and see 12:00:00.  Then
 9.01 seconds later, at 12:00:10.008 you try to read it back.  Memoize
 will look at the clock and see 12:00:10 and conclude that the value
-has expired.  This will probably not occur if if you have
+has expired.  This will probably not occur if you have
 C<Time::HiRes> installed.
 
 =head1 AUTHOR
 
-Mark-Jason Dominus (mjd-perl-memoize+@plover.com)
+Mark-Jason Dominus
 
 Mike Cariaso provided valuable insight into the best way to solve this
 problem.
@@ -360,11 +364,4 @@ perl(1)
 
 The Memoize man page.
 
-http://www.plover.com/~mjd/perl/Memoize/  (for news and updates)
-
-I maintain a mailing list on which I occasionally announce new
-versions of Memoize.  The list is for announcements only, not
-discussion.  To join, send an empty message to
-mjd-perl-memoize-request@Plover.com.
-
 =cut
