@@ -1,6 +1,8 @@
 package Memoize::SDBM_File;
+
 use SDBM_File;
 @ISA = qw(SDBM_File);
+$VERSION = '1.04';
 
 $Verbose = 0;
 
@@ -61,3 +63,17 @@ sub STORE {
 # Inherit FETCH and TIEHASH
 
 1;
+
+__END__
+
+=pod
+
+=head1 NAME
+
+Memoize::SDBM_File - glue to provide EXISTS for SDBM_File for Storable use
+
+=head1 DESCRIPTION
+
+See L<Memoize>.
+
+=cut
