@@ -1,6 +1,7 @@
-
 package Memoize::Storable;
+
 use Storable ();
+$VERSION = '1.04';
 $Verbose = 0;
 
 sub TIEHASH {
@@ -58,4 +59,19 @@ sub FIRSTKEY {
 sub NEXTKEY {
   undef;
 }
+
 1;
+
+__END__
+
+=pod
+
+=head1 NAME
+
+Memoize::Storable - store Memoized data in Storable database
+
+=head1 DESCRIPTION
+
+See L<Memoize>.
+
+=cut
